/*
 * Ad9833.c
 *
 *  Created on: 30 Aug 2021
 *      Author: Andrea
 */

#include "main.h"
#include "Ad9833.h"

//+++ Functions.
void AD9833Write(uint16_t SpiWord, SpiDevice SpiD)
{
	HAL_GPIO_WritePin(SpiD.ChipSelPort, SpiD.ChipSelPin, GPIO_PIN_RESET);
	HAL_StatusTypeDef status = HAL_SPI_Transmit_IT(SpiD.pHspi, (uint8_t*) &SpiWord, 1);
	while (status != HAL_OK);	//+++ Harsh.
	HAL_GPIO_WritePin(SpiD.ChipSelPort, SpiD.ChipSelPin, GPIO_PIN_SET);
}

void AD9833SetWave(uint16_t Wave, SpiDevice SpiD)
{
	switch (Wave)
	{
	case 0:
		AD9833Write(0x2000, SpiD); //+++ Sinusoidal wave.
		break;
	case 1:
		AD9833Write(0x2028, SpiD); //+++ Square wave.
		break;
	case 2:
		AD9833Write(0x2002, SpiD); //+++ Triangle wave.
		break;
	default:
		break;
	}
}

void AD9833SetWaveData(uint16_t Wave, float Frequency, float Phase,
		SpiDevice SpiD)
{
	uint16_t freqLW, freqHW;    //+++ MSB and LSB of frequency tuning word.
	uint32_t phaseVal;  //+++ Phase tuning value.

	//+++ Tuning word for phase: 0 - 360 degree.
	if (Phase < 0)
		Phase = 0; //+++ Changing phase value to positive.
	if (Phase > 360)
		Phase = 360; //+++ Maximum value for phase.

	/**
	 * 4096/360 = 11.37 change per degree for register and using 0xC000 which
	 * is phase 0 register address.
	 */
	phaseVal = ((int) (Phase * (4096 / 360))) | 0XC000;

	//+++ Tuning word for frequency.
	long freq = (int) (((Frequency * pow(2, 28)) / FMCLK) + 1); //+++ Tuning word.
	freqHW = (int) ((freq & 0xFFFC000) >> 14);
	freqLW = (int) (freq & 0x3FFF);
	freqLW |= 0x4000;
	freqHW |= 0x4000;

	//+++ Writing data.
	AD9833Write(0x2100, SpiD); //+++ Enable 16bit words and set reset bit.
	AD9833Write(freqLW, SpiD);
	AD9833Write(freqHW, SpiD);
	AD9833Write(phaseVal, SpiD);
	AD9833Write(0x2000, SpiD); //+++ Clear reset bit.
	AD9833SetWave(Wave, SpiD);
}

void HAL_SPI_ErrorCallback(SPI_HandleTypeDef *hspi)
{
	HAL_SPI_StateTypeDef state = HAL_SPI_GetState(hspi);
	while (state == HAL_SPI_STATE_ERROR);	//+++ Harsh.
	uint32_t error = HAL_SPI_GetError(hspi);
	while (error != HAL_SPI_ERROR_NONE);	//+++ Harsh.
}

void HAL_SPI_TxCpltCallback(SPI_HandleTypeDef *hspi)
{
	HAL_SPI_StateTypeDef state = HAL_SPI_GetState(hspi);
	while (state == HAL_SPI_STATE_ERROR);	//+++ Harsh.
}
